@import '~@synerty/peek-plugin-base-js/src/styles/_mixins';
@import '~@synerty/peek-plugin-base-js/src/styles/_variables';
@import "src/variables.scss";

$panel_width: 400px;

$edit_toolbar_button_height: 20px;
$edit_toolbar_height: $edit_toolbar_button_height + 2 * $panel_width;


.peek-tabs-section {
    width: unset;
}


.close {
    padding: 12px;
}

.peek-tabs-section-body {
    padding: $general_padding_1;
    top: $title_bar_height;
}


/* The side navigation menu */
.edit-props-panel {
    position: absolute; /* Stay in place */
    z-index: 4; /* Stay on top */
    //top: 46px;
    right: 8px;
    top: 8px;
    overflow-x: hidden; /* Disable horizontal scroll */
    transition: 0.5s; /* 0.5 second transition effect to slide in the toolbar */
    background-color: white;
    margin-bottom: 0;
    border-radius: 2px;
}

.edit-props-panel > .panel-header {
    display: flex;
    align-items: center;
    justify-content: space-between;
    padding: 15px;
    background: $primary-color;

    > * {
        color: #fff;
        fill: #fff;
        opacity: 1;
        margin: 0;
    }

    button {
        padding: 0;
        font-size: 20px;
    }
}

.edit-props-panel > .close {
    padding: 6px;
}

/* Set the style for when the toolbar is open */
.edit-props-is-open {
    min-width: 60px;
    max-width: 50vw;
}

/* Set the style for when the toolbar is closed */
.edit-props-is-closed {
    width: 0;
    padding-right: 0;
}

//
///* On smaller screens, where width is less than 450px,
//    change the style of the toolbar (less padding and a smaller font size) */
//@media screen and (max-width: 450px) {
//  .edit-props-is-open {
//    max-width: 70vw;
//  }
//}
