@import '~@synerty/peek-plugin-base-js/src/styles/_mixins';
@import '~@synerty/peek-plugin-base-js/src/styles/_variables';

.invert-image {
    -webkit-filter: contrast(1) hue-rotate(202deg) saturate(9.9) invert(1);
    filter: contrast(1) hue-rotate(202deg) saturate(9.9) invert(1);
}


.fill-screen {
    float: left;
    top: 0px;
    bottom: 0px;
    left: 0px;
    right: 0px;
    background-color: white;
    color: black;
    z-index: 500;
    position: absolute;

}


@media print {
    .no-print {
        display: none;
    }
}
